import { FastifyPluginAsync } from 'fastify';
import Ajv, { JSONSchemaType, type Schema } from 'ajv';
import crypto from 'node:crypto';

import {
    InvalidAuthorization,
    InvalidPayload,
    QuotaExceeded,
    ValidationError,
} from '../errors.js';
import { config } from '../config.js';
import { save } from '../store.js';

type UploadPayload = {
    data: string; // base64-encoded
    name: string;
};
const ajv = new Ajv();
const upload_schema = {
    type: 'object',
    properties: {
        data: { type: 'string' },
        name: { type: 'string' },
    },
    required: ['data', 'name'],
} as Schema | JSONSchemaType<UploadPayload>;

export default (async function (fastify) {
    fastify.post('/upload', async (req) => {
        const sha = crypto.createHash('sha256');
        sha.update(req.headers.authorization?.replace('Bearer ', '') ?? 'none');
        const key = sha.digest().toString('hex');

        const client = config.clients.find((x) => x.keySha256Sum === key);

        if (!client) {
            throw InvalidAuthorization();
        }

        const body = req.body as UploadPayload;

        if (typeof body !== 'object') {
            throw InvalidPayload();
        }

        const compiled = ajv.compile(upload_schema);
        if (!compiled(body)) {
            throw ValidationError(
                compiled.errors
                    ?.map((x) => x.keyword + ': ' + x.message)
                    .join('; '),
            );
        }

        const data = Buffer.from(body.data, 'base64');
        const name = body.name;

        const status = await save(data, name, client.name);
        if (status === 'Quota exceeded') {
            throw QuotaExceeded();
        }

        return {
            status: 'ok',
            name,
        };
    });
} as FastifyPluginAsync);
